#include  <stdio.h>
#include  <stdlib.h>
#include <unistd.h>
#include  <string.h>
#include  <time.h>

#include  <tthimage.h>

#include  "funcs.h"

#define SHIFTER		1
#define ONE_DIM		(256>>SHIFTER)
#define T_TABLEAU	(ONE_DIM*ONE_DIM*ONE_DIM)

long		tableau[ONE_DIM][ONE_DIM][ONE_DIM];

/* ----------------------------------------------------------------- */
int mk_colorcube(Image_Desc *img, char *outname)
{
long		t_tableau, lval, maxtable;
int		x, y, r, g, b;
FILE		*fp;
int		nbre;
double		echelle;
char		chaine[100];

fprintf(stderr, "shifter     %d\n", SHIFTER);
fprintf(stderr, "one_dim     %d\n", ONE_DIM);
t_tableau = T_TABLEAU * sizeof(long);
fprintf(stderr, "t_tableau   %d long -> %ld bytes\n", T_TABLEAU, t_tableau);

if (NULL==(fp=fopen(outname, "w"))) {
	perror(outname);
	exit(1);
	}

fprintf(fp, "/****** generated file ******/\n");
	
fprintf(fp, "#declare ccube = object\n{\n");
fprintf(fp, "union {\n");

maxtable = 0L;
memset((void *)tableau, 0, T_TABLEAU*sizeof(long));

for (y=0; y<IMG_HEIGHT(img); y++) {
	for (x=0; x<IMG_WIDTH(img); x++) {
		Image_getRGB(img, x, y, &r, &g, &b);
		// fprintf(stderr, "%4d %4d : %3d %3d %3d\n", x, y, r, g, b);
		r >>= SHIFTER;
		g >>= SHIFTER;
		b >>= SHIFTER;		
		tableau[r][g][b]++;
		if (tableau[r][g][b] > maxtable)
			maxtable = tableau[r][g][b];
		}
	}

fprintf(stderr, "maxtable = %ld\n", maxtable);

#define OBJECT  " sphere <0, 0, 0>, DS "	

nbre = 0;
echelle = 0.0;

for (r=0; r<ONE_DIM; r++) {
    for (g=0; g<ONE_DIM; g++) {
        for (b=0; b<ONE_DIM; b++) {
	
	    lval = tableau[r][g][b];
	
	    if (lval > 2L) {
	    
	    	sprintf(chaine, "pigment { rgb <%f, %f, %f> }",
			(double)r/63.001, (double)g/63.001, (double)b/63.001);
			
	        fprintf(fp, "object { S %s translate <%d, %d, %d> }\n",
				chaine,
				r<<SHIFTER, g<<SHIFTER, b<<SHIFTER);
	
		nbre++;
		}
	    }
	}
    }

fprintf(fp, " } finish { phong 0.70 }\n}\n/* nbre %d  */\n",
							     nbre);
fclose(fp);

fprintf(stderr, "%d values plotted\n", nbre);

return -1;
}
/* ----------------------------------------------------------------- */
int main (int argc, char *argv[])
{
Image_Desc      *src;
char		*srcname, *incname;
int		foo;

#if DEBUG_LEVEL
Image_print_version(1);
#endif

if (argc != 3)
	{
	fprintf(stderr, "%s : il manque les noms des fichiers\n", argv[0]);
	return 1;
	} 

srcname = argv[1];
incname = argv[2];

srand48(time(NULL));		/* wtf ? */

if (NULL==(src=Image_TGA_alloc_load(srcname)))
        {
        fprintf(stderr, "'%s' is not readable.\n", srcname);
        exit(1);
        }

foo = mk_colorcube(src, incname);
fprintf(stderr, "retour mk ccube = %d\n", foo);

Image_egalise_RGB(src, src, 0);
Image_TGA_save("plane_cc.tga", src, 0);

return 0;
}
